/****************************************************************************
 *   Copyright (C) 2009-2010 by Claas Anders "CaScAdE" Rathje               *
 *   admiralcascade@gmail.com                                               *
 *   Project-URL: http://www.mylifesucks.de/oss/c-osd/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "main.h"
#include "max7456_software_spi.h"
#include "osd_helpers.h"
#include "osd_ncmode_default.h"

#if !(ALLCHARSDEBUG|(WRITECHARS != -1))

int osd_ncmode_default() {
    uint8_t rc_signal = naviData.RC_RSSI ? naviData.RC_RSSI : naviData.RC_Quality; // if RSSI is present use it, else use Qality

    if (COSD_FLAGS_MODES & COSD_FLAG_HUD) {
        // write icons at init or after menu/mode-switch
        if (!(COSD_FLAGS_RUNTIME & COSD_ICONS_WRITTEN)) {
            if (COSD_FLAGS_CONFIG & COSD_FLAG_FEET) {
                write_char_xy(5, top_line, 0x7D); // mph
                write_char_xy(27, top_line + 1, 0x7E); // small feet ft home
                write_char_xy(27, top_line, 0x7E); // small feet ft height
            } else {
                write_char_xy(5, top_line, 0xCB); // km/h
                write_char_xy(27, top_line + 1, 0xCC); // small meters m home
                write_char_xy(27, top_line, 0xCC); // small meters m height
            }

            write_char_xy(10, top_line, 0xCA); // RC-transmitter
            write_char_xy(16, top_line, 0xD0); // degree symbol

            write_char_xy(20, top_line + 1, 0xB0); // left circle
            write_char_xy(22, top_line + 1, 0xB2); // right circle


            write_char_xy(7, bottom_line, 0x9E); // small V
            if ((COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) || (COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT)) {
                write_char_xy(7, bottom_line - 1, 0x9F); // small A
                write_char_xy(14, bottom_line - 1, 0xB5); // mah
                if (COSD_FLAGS_MODES & COSD_FLAG_STROMVOLT) {
                    write_char_xy(21, bottom_line - 1, 0x9E); // small V
                }
            }
            write_char_xy(14, bottom_line, 0xD1); // on clock
            write_char_xy(21, bottom_line, 0xD2); // fly clock
            write_char_xy(26, bottom_line, 0xC8); // sat1
            write_char_xy(27, bottom_line, 0xC9); // sat2
            COSD_FLAGS_RUNTIME |= COSD_ICONS_WRITTEN;
        }

        // first line
        if (COSD_FLAGS_CONFIG & COSD_FLAG_FEET) {
            // experimental cm/s -> mph
            write_ndigit_number_u(2, top_line, (uint16_t)(((uint32_t)naviData.GroundSpeed * (uint32_t)279) / (uint32_t)12500), 3, 0);
        } else {
            write_ndigit_number_u(2, top_line, (uint16_t)(((uint32_t)naviData.GroundSpeed * (uint32_t)9) / (uint32_t)250), 3, 0);
        }

        write_ndigit_number_u(7, top_line, rc_signal, 3, 0);
        if (rc_signal <= RCLVL_WRN && last_RC_Quality > RCLVL_WRN) {
            for (uint8_t x = 0; x < 4; x++)
                write_char_att_xy(7 + x, top_line, BLINK);
        } else if (rc_signal > RCLVL_WRN && last_RC_Quality <= RCLVL_WRN) {
            for (uint8_t x = 0; x < 4; x++)
                write_char_att_xy(7 + x, top_line, 0);
        }


        if (naviData.NCFlags & NC_FLAG_NOSERIALLINK) {
            write_char_xy(11, top_line, 0); // clear
        } else {
            write_char_xy(11, top_line, 0xC6); // PC icon
        }

        write_ndigit_number_u(13, top_line, naviData.CompassHeading, 3, 0);

        write_ascii_string_pgm(17, top_line, (const char *)(pgm_read_word(&(directions[heading_conv(naviData.CompassHeading)]))));

        draw_variometer(21, top_line, naviData.Variometer);

        if (COSD_FLAGS_CONFIG & COSD_FLAG_GPSHEIGHT) {
            if (COSD_FLAGS_CONFIG & COSD_FLAG_FEET) {
                // feet
                write_ndigit_number_s(23, top_line, (int16_t)(naviData.CurrentPosition.Altitude / 1000 - altimeter_offset) * 32 / 10, 4, 0); // GPS
            } else {
                if (naviData.CurrentPosition.Altitude / 1000 - altimeter_offset > 10 || naviData.CurrentPosition.Altitude / 1000 - altimeter_offset < -10) {
                    // above 10m only write full meters
                    write_ndigit_number_s(23, top_line, (int16_t)(naviData.CurrentPosition.Altitude / 1000 - altimeter_offset), 4, 0); // GPS
                } else {
                    // up to 10m write meters.dm
                    write_ndigit_number_s_10th(23, top_line, (int16_t)(naviData.CurrentPosition.Altitude / 100 - altimeter_offset * 10), 3, 0); // GPS
                }
            }
        } else {
            if (COSD_FLAGS_CONFIG & COSD_FLAG_FEET) {
                write_ndigit_number_s(23, top_line, naviData.Altimeter / 10 * 32 / 30, 4, 0); // BARO
            } else {
                //note:lephisto:according to several sources it's /30
                if (naviData.Altimeter > 300 || naviData.Altimeter < -300) {
                    // above 10m only write full meters
                    write_ndigit_number_s(23, top_line, naviData.Altimeter / 30, 4, 0); // BARO
                } else {
                    // up to 10m write meters.dm
                    write_ndigit_number_s_10th(23, top_line, naviData.Altimeter / 3, 3, 0); // BARO
                }
            }
        }


        // seccond line
        draw_compass(11, top_line + 1, naviData.CompassHeading);

        // TODO: verify correctness
        uint16_t heading_home = (naviData.HomePositionDeviation.Bearing + 360 - naviData.CompassHeading) % 360;
        //write_char_xy(21, top_line + 1, arrowdir[heading_conv(heading_home)]);
        // finer resolution, 0xa0 is first character and we add the index 0 <= index < 16
        write_char_xy(21, top_line + 1, 0xa0 + heading_fine_conv(heading_home));

        if (COSD_FLAGS_CONFIG & COSD_FLAG_FEET) {
            // feet
            write_ndigit_number_u(23, top_line + 1, naviData.HomePositionDeviation.Distance / 10 * 32 / 10, 4, 0);
        } else {
            write_ndigit_number_u(23, top_line + 1, naviData.HomePositionDeviation.Distance / 10, 4, 0);
        }

        // center
        if (naviData.FCFlags & FLAG_MOTOR_RUN) { // should be engines running
            if (!(old_MKFlags & FLAG_MOTOR_RUN)) { // motors just started, clear middle
                clear();
                // remember current heigth for gps offset
                altimeter_offset = naviData.CurrentPosition.Altitude / 1000;
                // set wasted counter to current offset
                if ((COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) && !(COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT)) {
                    wasted_ampere_offset = ampere_wasted / 10;
                } else if (COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT) {
                    wasted_ampere_offset = naviData.UsedCapacity;
                }
                // update flags to paint display again if needed
                COSD_FLAGS_RUNTIME &= ~COSD_ICONS_WRITTEN;
            }
            if (COSD_FLAGS_MODES & COSD_FLAG_ARTHORIZON) { // horizon
                uint8_t horizon_bottom = bottom_line - 1;
                if ((COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) || (COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT)) {
                    horizon_bottom--;
                }
                if (COSD_FLAGS_MODES & COSD_FLAG_AGGRHORIZON) {
                    draw_agressiva_artificial_horizon(top_line + 2, horizon_bottom, naviData.AngleNick, naviData.AngleRoll);
                } else {
                    draw_artificial_horizon(top_line + 2, horizon_bottom, naviData.AngleNick, naviData.AngleRoll);
                }
            }
            // motors are on, assume we were/are flying
            COSD_FLAGS_RUNTIME |= COSD_WASFLYING;
        } else {
            if ((old_MKFlags & FLAG_MOTOR_RUN)) { // motors just stopped
                clear(); // clear whole screen in case there is horizon stuff left
                // update flags to paint display again if needed
                COSD_FLAGS_RUNTIME &= ~COSD_ICONS_WRITTEN;
            }
            // stats
            if ((COSD_FLAGS_RUNTIME & COSD_WASFLYING) && (COSD_FLAGS_MODES & COSD_FLAG_STATS)) {
                uint8_t line = 3;
                write_ascii_string_pgm(1, line, (const char *)(pgm_read_word(&(stats_item_pointers[0])))); // max Altitude
                write_ascii_string_pgm(1, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[1])))); // max Speed
                write_ascii_string_pgm(1, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[2])))); // max Distance

                if (COSD_FLAGS_CONFIG & COSD_FLAG_FEET) {
                    write_ndigit_number_s(16, line - 2, max_Altimeter * 32 / 10, 4, 0);
                    write_char_xy(20, line - 2, 0x7E); // small feet ft
                    write_ndigit_number_u(17, line - 1, (uint16_t)(((uint32_t)max_GroundSpeed * (uint32_t)279) / (uint32_t)12500), 3, 0);
                    write_char_xy(20, line - 1, 0x7D); // mp/h
                    write_ndigit_number_u(17, line - 0, max_Distance / 10 * 32 / 10, 3, 0);
                    write_char_xy(20, line - 0, 0x7E); // small feet ft
                } else {
                    write_ndigit_number_s(16, line - 2, max_Altimeter, 4, 0);
                    write_char_xy(20, line - 2, 204); // small meters m
                    write_ndigit_number_u(17, line - 1, (uint16_t)(((uint32_t)max_GroundSpeed * (uint32_t)9) / (uint32_t)250), 3, 0);
                    write_char_xy(20, line - 1, 203); // km/h
                    write_ndigit_number_u(17, line - 0, max_Distance / 10, 3, 0);
                    write_char_xy(20, line - 0, 204); // small meters m
                }

                write_ascii_string_pgm(1, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[3])))); // min voltage
                write_ndigit_number_u_10th(16, line, min_UBat, 3, 0);
                write_char_xy(20, line, 0x9E); // small V
                if ((COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) || (COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT)) {
                    write_ascii_string_pgm(1, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[7])))); // ampere
                    write_ndigit_number_u_10th(16, line, max_ampere / 10, 3, 0);
                    write_char_xy(20, line, 0x9F); // small A

                    // wasted mampere in this flight (will count up after landing)
                    if ((COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) && !(COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT)) {
                        write_ndigit_number_u(21, line, (ampere_wasted / 10) - wasted_ampere_offset, 5, 0);
                    } else if (COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT) {

                        write_ndigit_number_u(21, line, naviData.UsedCapacity - wasted_ampere_offset, 5, 0);
                    }

                    write_char_xy(26, line, 0xB5); // mah
                }
                write_ascii_string_pgm(1, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[4])))); // max time
                write_time(14, line, max_FlyingTime);
                write_char_xy(20, line, 210); // fly clock
                write_ascii_string_pgm(1, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[5])))); // longitude
                write_gps_pos(14, line, naviData.CurrentPosition.Longitude);
                write_ascii_string_pgm(1, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[6])))); // latitude
                write_gps_pos(14, line, naviData.CurrentPosition.Latitude);
            } else if (COSD_FLAGS_MODES & COSD_FLAG_ARTHORIZON) { // if no stats there is space horizon
                uint8_t horizon_bottom = bottom_line - 1;
                if ((COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) || (COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT)) {
                    horizon_bottom--;
                }
                if (COSD_FLAGS_MODES & COSD_FLAG_AGGRHORIZON) {
                    draw_agressiva_artificial_horizon(top_line + 2, horizon_bottom, naviData.AngleNick, naviData.AngleRoll);
                } else {
                    draw_artificial_horizon(top_line + 2, horizon_bottom, naviData.AngleNick, naviData.AngleRoll);
                }
            }
        }
        if (COSD_FLAGS_MODES & COSD_FLAG_BIGVARIO) {
            draw_big_variometer(27, 8, naviData.Variometer);
        }

        // pre-bottom line
        if ((COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) && !(COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT)) {
            //write_ndigit_number_s(3, bottom_line - 1, ampere, 4, 0);
            write_ndigit_number_u_10th(3, bottom_line - 1, ampere / 10, 3, 0);
            write_ndigit_number_s(10, bottom_line - 1, ampere_wasted / 10, 4, 0);
            if (COSD_FLAGS_MODES & COSD_FLAG_STROMVOLT) {
                write_ndigit_number_u_10th(17, bottom_line - 1, s_volt, 3, 0);
            }
        } else if (COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT) {
            write_ndigit_number_u_10th(3, bottom_line - 1, naviData.Current, 3, 0);
            write_ndigit_number_u(10, bottom_line - 1, naviData.UsedCapacity, 4, 0);
        }

        //DEBUGwrite_ndigit_number_u(1, 5, COSD_FLAGS_MODES, 3, 0);

        // bottom line
        draw_battery(2, bottom_line, min_voltage, naviData.UBat, max_voltage);
        write_ndigit_number_u_10th(3, bottom_line, naviData.UBat, 3, 0);
        if (naviData.UBat <= min_voltage && last_UBat > min_voltage) {
            for (uint8_t x = 2; x < 8; x++)
                write_char_att_xy(x, bottom_line, BLINK);
        } else if (naviData.UBat > min_voltage && last_UBat < min_voltage) {
            for (uint8_t x = 2; x < 8; x++)
                write_char_att_xy(x, bottom_line, 0);
        }

        write_time(8, bottom_line, uptime);
        write_time(15, bottom_line, naviData.FlyingTime);

        write_ndigit_number_u(24, bottom_line, naviData.SatsInUse, 2, 0);

        if (naviData.NCFlags & NC_FLAG_MANUAL_CONTROL) {
            write_char_xy(23, bottom_line, 0xB3); // rc transmitter
        } else {
            write_char_xy(23, bottom_line, 0); // clear
        }

        if (naviData.NCFlags & NC_FLAG_CH) {
            write_char_xy(27, bottom_line, 231); // gps ch
        } else if (naviData.NCFlags & NC_FLAG_PH) {
            write_char_xy(27, bottom_line, 230); // gps ph
        } else { // (naviData.NCFlags & NC_FLAG_FREE)
            write_char_xy(27, bottom_line, 201); // sat2 (free)
        }
    }

    //write_number_s(8, 5, RxDataLen);
    //write_number_s(16, 5, setsReceived++);

    // remember statistics (only when engines running)
    if (naviData.FCFlags & FLAG_MOTOR_RUN) {
        if (COSD_FLAGS_CONFIG & COSD_FLAG_GPSHEIGHT) {
            if (naviData.CurrentPosition.Altitude / 1000 - altimeter_offset > max_Altimeter) max_Altimeter = naviData.CurrentPosition.Altitude / 1000;
        } else {
            if (naviData.Altimeter / 30 > max_Altimeter) max_Altimeter = naviData.Altimeter / 30;
        }
        if (naviData.GroundSpeed > max_GroundSpeed) max_GroundSpeed = naviData.GroundSpeed;
        if (naviData.HomePositionDeviation.Distance > max_Distance) {
            max_Distance = naviData.HomePositionDeviation.Distance;
        }
        if (naviData.UBat < min_UBat) min_UBat = naviData.UBat;
        if (naviData.FlyingTime > max_FlyingTime) max_FlyingTime = naviData.FlyingTime;
        if ((COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) && !(COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT)) {
            if (ampere > max_ampere) max_ampere = ampere;
        } else if (COSD_FLAGS_MODES & COSD_FLAG_FCCURRENT) {
            if (naviData.Current * 10 > max_ampere) max_ampere = naviData.Current * 10;
        }
    }

    // remember last values
    last_RC_Quality = rc_signal;
    last_UBat = naviData.UBat;
    old_MKFlags = naviData.FCFlags;
    seconds_since_last_data = 0;

    return 0;
}

#endif
